<?php

namespace Drupal\s3fs_file_proxy_to_s3;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManager;
use Drupal\stage_file_proxy\DownloadManagerInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\ClientException;

/**
 * Override file fetching to pull from S3.
 */
class S3fsFileProxyToS3FetchManager implements DownloadManagerInterface {

  /**
   * Public file scheme.
   */
  private const PUBLIC_SCHEME = 'public://';

  public function __construct(
    protected ClientInterface $client,
    protected FileSystemInterface $fileSystem,
    protected ConfigFactoryInterface $configFactory,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public function fetch($server, $remote_file_dir, $relative_path, array $options): bool {
    try {
      // Fetch remote file.
      $url = $server . '/' . UrlHelper::encodePath($remote_file_dir . '/' . $relative_path);
      $response = $this->client->get($url, $options);

      if ($response->getStatusCode() === 200) {
        // Prepare local target directory and save downloaded file.
        $target_dir = static::PUBLIC_SCHEME . dirname($relative_path);
        if ($this->fileSystem->prepareDirectory($target_dir, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS)) {
          file_put_contents(static::PUBLIC_SCHEME . $relative_path, $response->getBody());
          return TRUE;
        }
      }
      return FALSE;
    }
    catch (ClientException $e) {
      // Do nothing.
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function filePublicPath(): string {
    return 's3fs_to_s3/files';
  }

  /**
   * {@inheritdoc}
   */
  public function styleOriginalPath($uri, $style_only = TRUE) {
    $scheme = StreamWrapperManager::getScheme($uri);
    if ($scheme) {
      $path = StreamWrapperManager::getTarget($uri);
    }
    else {
      $path = $uri;
      $scheme = $this->configFactory->get('system.file')->get('default_scheme');
    }

    // It is a styles path, so we extract the different parts.
    if (str_starts_with($path, 's3fs_to_s3/files/styles')) {
      // Then the path is like styles/[style_name]/[schema]/[original_path].
      return preg_replace('/s3fs_to_s3\/files\/styles\/.*\/(.*)\/(.*)/U', '$1://$2', $path);
    }

    // It is a styles path, so we extract the different parts.
    if (str_starts_with($path, 'styles')) {
      // Then the path is like styles/[style_name]/[schema]/[original_path].
      return preg_replace('/styles\/.*\/(.*)\/(.*)/U', '$1://$2', $path);
    }
    // Else it seems to be the original.
    elseif (!$style_only) {
      return "$scheme://$path";
    }
    else {
      return FALSE;
    }
  }

}
